"""peripheral methods that are unrelated to bogosort algorithm"""
import random

def generate_array(length):
    """
    modifies passed in array to
    return a random permutation
    """
    result = []
    for _ in range(length):
        result.append(random.randint(0,64))
    return result



"""
classic bogo sort algorithm split into
parts that are reused across files
"""
import random

def bogosort(arr):
    """
    returns new bogosorted array
    without modifying passed in one
    """
    result = list(arr)
    is_sorted = False
    while not is_sorted:
        shuffle(result)
        is_sorted = check_if_ordered(result)
    return result


def check_if_ordered(arr):
    """
    returns a bool value
    of whether the passed in array is in order or not
    """
    for i in range(len(arr)-1):
        if arr[i] > arr[i+1]:
            return False
    return True


def shuffle(arr):
    """
    modifies passed in array to
    return a random permutation
    """
    for i in range(len(arr)-1, 0, -1):
        j = random.randint(0, i)
        arr[i], arr[j] = arr[j], arr[i]

if __name__ == '__main__':
    bogosort(sys.argv[1])



"""runs the bogosort algorithm except multithreaded"""

import sys
import multiprocessing
import shitsort.peripherals
from shitsort.bogo import bogosort

def main(args):
    """Runs the sorting algorithm"""
    length = int(args[0])
    numthreads = int(args[1])
    oarray = peripherals.generate_array(length)

    print(multibogo(oarray, numthreads))


def multibogosort(arr, numthreads):
    """
    multithreads the bogosort procedure
    without modifying the passed in array
    """
    processes = []

    queue = multiprocessing.Queue()
    for _ in range(numthreads):

        # pass array as tuple because pickles ?? idk
        process = multiprocessing.Process(target=bogothread, args=(arr,queue))
        process.start()
        processes.append(process)

    # waits for one process to put a result in the queueueue
    result = queue.get()

    # then kills all of them
    # idc how you're meant to do it
    for process in processes:
        process.terminate()
        process.join()

    return result


def bogothread(arr, queue):
    """
    sorts array and queueueues it
    without modifying the passed in array
    """
    result = bogosort(arr)
    queue.put(result) # place result in queueueue when it is found

if __name__ == '__main__':
    main(sys.argv[1:])
    # length of array, number of threads
